﻿// THE MIT OPEN SOURCE LICENCE
// ===========================
// {Your website/product} uses the VAT Registration Number Validator by <link-to>mbbsoftware.com</link>.
//
// Permission is hereby granted, free of charge, to any person obtaining a copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the Software is furnished to do so, subject to the following conditions:
//
// The above copyright notice and this permission notice shall be included in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, 
// FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER 
// LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS 
// IN THE SOFTWARE.
//
// ==================================================================================================================================================================
//
//
// A note to the user
// ==================
// Dear User of the VAT Registration Number Validator,
//
// Thank you for obtaining a copy of this software and using it in your website(s) or other product(s).
//
// Since the above notice is included in the asp.net DLL assemblies and the C# code, it will not be visible by your 
// website visitors or software users. We would like to kindly ask you to help us by adding a link to our website on 
// your website, perhaps somewhere on one of your disclaimers or legal pages. For example, the following or similar 
// link would be much appreciated to be added to your site:
//
// {Your website/product} uses the VAT Registration Number Validator by <link-to>mbbsoftware.com</link>.
//
// This will greatly assist our visibility to search engines and will help us to develop more technology to benefit 
// the community of people using the Internet and the Internet as a whole, as well as helping us to make updates to 
// the VAT Registration Number Validator as needed.
//
// We thank you again for using the VAT Registration Number Validator, and especially if you will be so kind as to place 
// a link on your website to www.mbbsoftware.com. We wish you success with your website(s), product(s), and endeavors.
//
// Kind regards,
// MBBSoftware


using System;
using System.Configuration;
using System.Collections;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Web;
using System.IO;
using System.Net;
using System.Xml;
using System.Security.Cryptography;
using System.IO.IsolatedStorage;
using System.Diagnostics;
using System.Reflection;


[assembly:AssemblyTitle("The VAT Registration Number Validator library can validate VAT registration numbers with the European VAT database and acquire the VAT number registrant information.")]
[assembly:AssemblyDescription("The VAT Registration Number Validator library can validate VAT registration numbers with the European VAT database and acquire the VAT number registrant information.")]
[assembly:AssemblyConfiguration("Release")]
[assembly:AssemblyCompany("MBBSoftware")]
[assembly:AssemblyProduct("VAT Registration Number Validator")]
[assembly:AssemblyCopyright("Miroslav Bonchev Bonchev")]
[assembly:AssemblyTrademark("")]
[assembly:AssemblyCulture("")]
[assembly:AssemblyVersion("4.0.0.0")]
[assembly:AssemblyDelaySign(false)]
[assembly:AssemblyKeyFile("")]
[assembly:AssemblyKeyName("")]


namespace MBBSoftware
{
   public class VATValidator
   {
      private const string ErrorMessage0 = "Invalid input data.";
      private const string ErrorMessage1 = "Invalid or unrecognized VAT server response data. Please contact support.";
      private const string ErrorMessage2 = "Invalid Member State Language Code or VAT check DLL is outdated. Error 0x80000001.";
      private const string ErrorMessage3 = "Invalid Member State Language Code or VAT check DLL is outdated. Error 0x80000007.";
      private const string ErrorMessage4 = "Invalid Member State Code or VAT check DLL is outdated. Error 0x80000006.";
      private const string ErrorMessage5 = "Invalid Member State Code or VAT check DLL is outdated. Error 0x80000008.";
      private const string ErrorMessage6 = "VAT check DLL is outdated. Error 0x80000002.";
      private const string ErrorMessage7 = "There is no license present. Please supply a license using \"VATValidatorLicenseCode\" application settings variable in the application web.config.";
      private const string ErrorMessage8 = "VAT check DLL is outdated. Error 0x80000003.";
      private const string ErrorMessage9 = "VAT check DLL is outdated. Error 0x80000004.";
      private const string ErrorMessageA = "VAT check DLL is outdated. Error 0x80000005.";
      private const string ErrorMessageB = "Authentication exception. Error: {0}";
      private const string ErrorMessageC = "Authentication failed.";
      private const string ErrorMessageD = "The VAT validator dll experienced an exception while verifying your license. Please notify support@mbbsoftware.com if this error persists. Exception: {0}";
      private const string ErrorMessageE = "Unknown response code \"{0}\". Please contact support at support@mbbsoftware.com or use the latest VATValidator dll to prevent interruptions of the service.";
      private const string ErrorMessageF = "Could not update the local storage. Please resolve the problem, or contact support at support@mbbsoftware.com or to prevent interruptions of the service. Error: {0}";
      private const string ErrorMessageG = "Unknown response code \"{0}\" and message \"{1}\". Please contact support at support@mbbsoftware.com or use the latest VATValidator dll to prevent interruptions of the service.";
      private const string ErrorMessageH = "Unknown response error, and missing or expired license verification. Webmaster message is set, with content: \"{0}\"";
      private const string ErrorMessageJ = "Invalid Member State Language Code or VAT check DLL is outdated. Error 0x80000009.";


      public class InvalidParameter : Exception
      {
         public InvalidParameter( string Message ) : base( Message )
         {
         }
      }


      public class ServerException : Exception
      {
         public ServerException( string Message ) : base( Message )
         {
         }
      }


      public class ParsingException : Exception
      {
         public ParsingException( string Message ) : base( Message )
         {
         }
      }


      public enum Language
      {
         Bulgarian,  // bg
         Czech,      // cs
         Dansk,      // da
         Deutch,     // de
         Greek,      // el
         English,    // en
         Español,    // es
         Estonian,   // et
         Suomea,     // fi
         Français,   // fr
         Hrvatski,   // hr
         Hungarian,  // hu
         Italiano,   // it
         Lithuanian, // lt
         Latvian,    // lv
         Maltese,    // mt
         Nederlands, // nl
         Polish,     // pl
         Português,  // pt
         Romanian,   // ro
         Slovak,     // sk
         Slovenian,  // sl
         Svenska     // sv
      };

      
      public enum MemberState
      {
         Austria,          // AT
         Belgium,          // BE
         Bulgaria,         // BG
         Croatia,          // HR
         Cyprus,           // CY
         CzechRepublic,    // CZ
         Germany,          // DE
         Denmark,          // DK
         Estonia,          // EE
         Greece,           // EL
         Spain,            // ES
         Finland,          // FI
         France,           // FR
         UnitedKingdom,    // GB
         Hungary,          // HU
         Ireland,          // IE
         Italy,            // IT
         Lithuania,        // LT
         Luxembourg,       // LU
         Latvia,           // LV
         Malta,            // MT
         TheNetherlands,   // NL
         Poland,           // PL
         Portugal,         // PT
         Romania,          // RO
         Sweden,           // SE
         Slovenia,         // SI
         Slovakia          // SK
      };

      
      public enum DataItem
      {
         Status                = 0,
         MemberState           = 1,
         VATNumber             = 2,
         TimeRequest           = 3,
         Name                  = 4,
         Address               = 5,
         ConsultationReference = 6
      }


      private string RequesterStateCode;
      private string RequesterVATNumber;


      public VATValidator()
      {
      }


      public VATValidator( MemberState stateRequester, string vatRequester )
      {
         RequesterStateCode = GetMemberState( stateRequester );
         RequesterVATNumber = vatRequester;
      }


      public static string GetLanguage( Language language )
      {
         switch( language )
         {
         case Language.Bulgarian:  return( "bg" );
			case Language.Czech:      return( "cs" );
			case Language.Dansk:      return( "da" );
			case Language.Deutch:     return( "de" );
			case Language.Greek:      return( "el" );
			case Language.English:    return( "en" );
			case Language.Español:    return( "es" );
			case Language.Estonian:   return( "et" );
			case Language.Suomea:     return( "fi" );
			case Language.Français:   return( "fr" );
			case Language.Hrvatski:   return( "hr" );
			case Language.Hungarian:  return( "hu" );
			case Language.Italiano:   return( "it" );
			case Language.Lithuanian: return( "lt" );
			case Language.Latvian:    return( "lv" );
			case Language.Maltese:    return( "mt" );
			case Language.Nederlands: return( "nl" );
			case Language.Polish:     return( "pl" );
			case Language.Português:  return( "pt" );
			case Language.Romanian:   return( "ro" );
			case Language.Slovak:     return( "sk" );
			case Language.Slovenian:  return( "sl" );
			case Language.Svenska:    return( "sv" );
         }

         throw( new InvalidParameter( ErrorMessage2 ) );
      }


      public static Language GetLanguage( string languageCode )
      {
         switch( languageCode.ToLower() )
         {
         case "bg": return( Language.Bulgarian );
			case "cs": return( Language.Czech );
			case "da": return( Language.Dansk );
			case "de": return( Language.Deutch );
			case "el": return( Language.Greek );
			case "en": return( Language.English );
			case "es": return( Language.Español );
			case "et": return( Language.Estonian );
			case "fi": return( Language.Suomea );
			case "fr": return( Language.Français );
			case "hr": return( Language.Hrvatski );
			case "hu": return( Language.Hungarian );
			case "it": return( Language.Italiano );
			case "lt": return( Language.Lithuanian );
			case "lv": return( Language.Latvian );
			case "mt": return( Language.Maltese );
			case "nl": return( Language.Nederlands );
			case "pl": return( Language.Polish );
			case "pt": return( Language.Português );
			case "ro": return( Language.Romanian );
			case "sk": return( Language.Slovak );
			case "sl": return( Language.Slovenian );
			case "sv": return( Language.Svenska );
         }

         throw( new InvalidParameter( ErrorMessage3 ) );
      }


      public static string[] GetLanguages()
      {
         return( new string[] { "bg", "cs", "da", "de", "el", "en", "es", "et", "fi", "fr", "hu", "it", "lt", "lv", "mt", "nl", "pl", "pt", "ro", "sk", "sl", "sv" } );
      }


      public static string GetMemberState( MemberState state )
      {
         switch( state )
         {
         case MemberState.Austria:        return( "AT" );
         case MemberState.Belgium:        return( "BE" );
         case MemberState.Bulgaria:       return( "BG" );
         case MemberState.Croatia:        return( "HR" );
         case MemberState.Cyprus:         return( "CY" );
         case MemberState.CzechRepublic:  return( "CZ" );
         case MemberState.Germany:        return( "DE" );
         case MemberState.Denmark:        return( "DK" );
         case MemberState.Estonia:        return( "EE" );
         case MemberState.Greece:         return( "EL" );
         case MemberState.Spain:          return( "ES" );
         case MemberState.Finland:        return( "FI" );
         case MemberState.France:         return( "FR" );
         case MemberState.UnitedKingdom:  return( "GB" );
         case MemberState.Hungary:        return( "HU" );
         case MemberState.Ireland:        return( "IE" );
         case MemberState.Italy:          return( "IT" );
         case MemberState.Lithuania:      return( "LT" );
         case MemberState.Luxembourg:     return( "LU" );
         case MemberState.Latvia:         return( "LV" );
         case MemberState.Malta:          return( "MT" );
         case MemberState.TheNetherlands: return( "NL" );
         case MemberState.Poland:         return( "PL" );
         case MemberState.Portugal:       return( "PT" );
         case MemberState.Romania:        return( "RO" );
         case MemberState.Sweden:         return( "SE" );
         case MemberState.Slovenia:       return( "SI" );
         case MemberState.Slovakia:       return( "SK" );
         }

         throw( new InvalidParameter( ErrorMessage4 ) );
      }


      public static MemberState GetMemberState( string stateCode )
      {
         switch( stateCode.ToUpper() )
         {
         case "AT": return( MemberState.Austria );
         case "BE": return( MemberState.Belgium );
         case "BG": return( MemberState.Bulgaria );
         case "CY": return( MemberState.Cyprus );
         case "CZ": return( MemberState.CzechRepublic );
         case "DE": return( MemberState.Germany );
         case "DK": return( MemberState.Denmark );
         case "EE": return( MemberState.Estonia );
         case "EL": return( MemberState.Greece );
         case "ES": return( MemberState.Spain );
         case "FI": return( MemberState.Finland );
         case "FR": return( MemberState.France );
         case "GB": return( MemberState.UnitedKingdom );
         case "HR": return( MemberState.Croatia );
         case "HU": return( MemberState.Hungary );
         case "IE": return( MemberState.Ireland );
         case "IT": return( MemberState.Italy );
         case "LT": return( MemberState.Lithuania );
         case "LU": return( MemberState.Luxembourg );
         case "LV": return( MemberState.Latvia );
         case "MT": return( MemberState.Malta );
         case "NL": return( MemberState.TheNetherlands );
         case "PL": return( MemberState.Poland );
         case "PT": return( MemberState.Portugal );
         case "RO": return( MemberState.Romania );
         case "SE": return( MemberState.Sweden );
         case "SI": return( MemberState.Slovenia );
         case "SK": return( MemberState.Slovakia );
         }

         throw( new InvalidParameter( ErrorMessage5 ) );
      }


      public static string[] GetMemberStates()
      {
         return( new string[] { "AT", "BE", "BG", "CY", "CZ", "DE", "DK", "EE", "EL", "ES", "FI", "FR", "GB", "HU", "IE", "IT", "LT", "LU", "LV", "MT", "NL", "PL", "PT", "RO", "SE", "SI", "SK" } );
      }


      public static string GetItemName( DataItem eItem, Language eLanguage )
      {
         switch( eItem )
         {
         case DataItem.Status:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Статус" );
            case Language.Czech:       return( "Postavení" );
            case Language.Dansk:       return( "Status" );
            case Language.Deutch:      return( "Status" );
            case Language.Greek:       return( "Kατάσταση" );
            case Language.English:     return( "Status" );
            case Language.Español:     return( "Estado" );
            case Language.Estonian:    return( "Staatus" );
            case Language.Suomea:      return( "Tila" );
            case Language.Français:    return( "Statut" );
            case Language.Hrvatski:    return( "Status" );
            case Language.Hungarian:   return( "Állapot" );
            case Language.Italiano:    return( "Stato" );
            case Language.Lithuanian:  return( "padėtis" );
            case Language.Latvian:     return( "Status" );
            case Language.Maltese:     return( "Status" );
            case Language.Nederlands:  return( "Staat" );
            case Language.Polish:      return( "Status" );
            case Language.Português:   return( "Estado" );
            case Language.Romanian:    return( "Stare" );
            case Language.Slovak:      return( "Postavenie" );
            case Language.Slovenian:   return( "Status" );
            case Language.Svenska:     return( "Status" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );


         case DataItem.MemberState:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Държава-членка" );
            case Language.Czech:       return( "Členský stát" );
            case Language.Dansk:       return( "Medlemsstat" );
            case Language.Deutch:      return( "Mitgliedstaat" );
            case Language.Greek:       return( "Κράτος μέλος" );
            case Language.English:     return( "Member State" );
            case Language.Español:     return( "Estado miembro" );
            case Language.Estonian:    return( "Liikmesriigi" );
            case Language.Suomea:      return( "Jäsenvaltio" );
            case Language.Français:    return( "État Membre" );
            case Language.Hrvatski:    return( "Država članica" );
            case Language.Hungarian:   return( "Tagállam" );
            case Language.Italiano:    return( "Stato membro" );
            case Language.Lithuanian:  return( "Valstybė narė" );
            case Language.Latvian:     return( "Dalībvalsts" );
            case Language.Maltese:     return( "Stat Membru" );
            case Language.Nederlands:  return( "Lidstaat" );
            case Language.Polish:      return( "Państwo Członkowskie" );
            case Language.Português:   return( "Estado-Membro" );
            case Language.Romanian:    return( "Statul membru" );
            case Language.Slovak:      return( "Členský štát" );
            case Language.Slovenian:   return( "Država članica" );
            case Language.Svenska:     return( "Medlemsstat" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );


         case DataItem.VATNumber:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Номер по ДДС" );
            case Language.Czech:       return( "DIČ" );
            case Language.Dansk:       return( "Momsregistreringsnummer" );
            case Language.Deutch:      return( "MwSt-Nummer" );
            case Language.Greek:       return( "Αριθμός ΦΠΑ" );
            case Language.English:     return( "VAT Number" );
            case Language.Español:     return( "Número de IVA" );
            case Language.Estonian:    return( "KA Number" );
            case Language.Suomea:      return( "VAT-numero" );
            case Language.Français:    return( "Numéro de TVA" );
            case Language.Hrvatski:    return( "PDV id. broj" );
            case Language.Hungarian:   return( "Közösségi adószám" );
            case Language.Italiano:    return( "Partita IVA" );
            case Language.Lithuanian:  return( "PVM mokėtojo kodas" );
            case Language.Latvian:     return( "PVN reģistrācijas numurs" );
            case Language.Maltese:     return( "Numru tal-VAT" );
            case Language.Nederlands:  return( "BTW-nummer" );
            case Language.Polish:      return( "Numer VAT" );
            case Language.Português:   return( "Número de IVA" );
            case Language.Romanian:    return( "Numărul de înregistrare pentru TVA" );
            case Language.Slovak:      return( "IČ DPH" );
            case Language.Slovenian:   return( "Davčna številka" );
            case Language.Svenska:     return( "Momsregistreringsnummer" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );

         
         case DataItem.TimeRequest:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Дата на получаване на запитването" );
            case Language.Czech:       return( "Datum přijetí žádosti" );
            case Language.Dansk:       return( "Tidspunkt for anmodningens modtagelse" );
            case Language.Deutch:      return( "Zeitpunkt des Eingangs der Anfrage" );
            case Language.Greek:       return( "Ημερομηνία παραλαβής της αίτησης" );
            case Language.English:     return( "Check Request Timestamp" );
            case Language.Español:     return( "Fecha de recepción de la solicitud" );
            case Language.Estonian:    return( "Kontrollige Soovin ajatempel" );
            case Language.Suomea:      return( "Pyynnön vastaanottoaika" );
            case Language.Français:    return( "Date de réception de la demande" );
            case Language.Hrvatski:    return( "Datum primitka zahtjeva" );
            case Language.Hungarian:   return( "A lekérdezés dátuma" );
            case Language.Italiano:    return( "Orario di ricezione della richiesta" );
            case Language.Lithuanian:  return( "Prašymą patikrinti timestamp" );
            case Language.Latvian:     return( "Pārbaudiet Pieprasīt Laika zīmoga" );
            case Language.Maltese:     return( "Iċċekkja Timestamp Talba" );
            case Language.Nederlands:  return( "Tijdstip waarop de aanvraag werd ontvangen" );
            case Language.Polish:      return( "Data zapytania" );
            case Language.Português:   return( "Data de recepção do pedido" );
            case Language.Romanian:    return( "Data la care a fost primită solicitarea" );
            case Language.Slovak:      return( "Dátum prijatia žiadosti" );
            case Language.Slovenian:   return( "Preverite Zahtevaj Časovni žig" );
            case Language.Svenska:     return( "Tidpunkt då förfrågan inkom" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );

         
         case DataItem.Name:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Име" );
            case Language.Czech:       return( "Jméno" );
            case Language.Dansk:       return( "Navn" );
            case Language.Deutch:      return( "Name" );
            case Language.Greek:       return( "Όνομα" );
            case Language.English:     return( "Name" );
            case Language.Español:     return( "Nombre" );
            case Language.Estonian:    return( "Nimi" );
            case Language.Suomea:      return( "Nimi" );
            case Language.Français:    return( "Nom" );
            case Language.Hrvatski:    return( "Naziv" );
            case Language.Hungarian:   return( "Név" );
            case Language.Italiano:    return( "Nome" );
            case Language.Lithuanian:  return( "Pavadinimas" );
            case Language.Latvian:     return( "Nosaukums" );
            case Language.Maltese:     return( "Isem" );
            case Language.Nederlands:  return( "Naam" );
            case Language.Polish:      return( "Nazwa" );
            case Language.Português:   return( "Nome" );
            case Language.Romanian:    return( "Nume" );
            case Language.Slovak:      return( "Názov" );
            case Language.Slovenian:   return( "Ime" );
            case Language.Svenska:     return( "Namn" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );

            
         case DataItem.Address:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Адрес" );
            case Language.Czech:       return( "Adresa" );
            case Language.Dansk:       return( "Adresse" );
            case Language.Deutch:      return( "Adresse" );
            case Language.Greek:       return( "Διεύθυνση" );
            case Language.English:     return( "Address" );
            case Language.Español:     return( "Domicilio" );
            case Language.Estonian:    return( "Aadress" );
            case Language.Suomea:      return( "Osoite" );
            case Language.Français:    return( "Adresse" );
            case Language.Hrvatski:    return( "Adresa" );
            case Language.Hungarian:   return( "Cím" );
            case Language.Italiano:    return( "Indirizzo" );
            case Language.Lithuanian:  return( "Adresas" );
            case Language.Latvian:     return( "Adrese" );
            case Language.Maltese:     return( "Indirizz" );
            case Language.Nederlands:  return( "Adres" );
            case Language.Polish:      return( "Adres" );
            case Language.Português:   return( "Endereço" );
            case Language.Romanian:    return( "Adres" );
            case Language.Slovak:      return( "Adresa" );
            case Language.Slovenian:   return( "Naslov" );
            case Language.Svenska:     return( "Adress" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );

         
         case DataItem.ConsultationReference:
            switch( eLanguage )
            {
            case Language.Bulgarian:   return( "Идентификатор на запитването" );
            case Language.Czech:       return( "unikátní číslo konzultace" );
            case Language.Dansk:       return( "Forespørgselsnummer" );
            case Language.Deutch:      return( "Abfrage-Nummer" );
            case Language.Greek:       return( "Αριθμό επαλήθευσης" );
            case Language.English:     return( "Consultation Number" );
            case Language.Español:     return( "Número de Consulta" );
            case Language.Estonian:    return( "Viitenumber" );
            case Language.Suomea:      return( "Yksilöivä viitenumero" );
            case Language.Français:    return( "Numéro d'identification" );
            case Language.Hrvatski:    return( "Referentni broj" );
            case Language.Hungarian:   return( "Egyedi hivatkozási szám" );
            case Language.Italiano:    return( "Numero di Consultazione" );
            case Language.Lithuanian:  return( "Nuoroda skaičius" );
            case Language.Latvian:     return( "Konsultācijas skaits" );
            case Language.Maltese:     return( "Numru Konsultazzjoni" );
            case Language.Nederlands:  return( "Consultatienummer" );
            case Language.Polish:      return( "Identyfikator zapytania" );
            case Language.Português:   return( "Número de Consulta" );
            case Language.Romanian:    return( "Numărul de identificare a solicitării" );
            case Language.Slovak:      return( "Identifikátor žiadosti" );
            case Language.Slovenian:   return( "Posvetovanje številka" );
            case Language.Svenska:     return( "Referensnummer" );
            }
            throw( new InvalidParameter( ErrorMessageJ ) );
         }


         throw( new InvalidParameter( ErrorMessage6 ) );
      }


      public static List< DataItem > GetAllItems()
      {
         List< DataItem > listIds = new List< DataItem >();

         listIds.Add( DataItem.Status );
         listIds.Add( DataItem.MemberState );
         listIds.Add( DataItem.VATNumber );
         listIds.Add( DataItem.TimeRequest );
         listIds.Add( DataItem.Name );
         listIds.Add( DataItem.Address );
         listIds.Add( DataItem.ConsultationReference );

         return( listIds );
      }


      public bool IsValidVAT( string strStateCode, string strVATNumber, string strLanguageCode, ref Dictionary< DataItem, string > dictResults, ref string WebMasterMessage )
      {
         return( IsValidVAT( GetMemberState( strStateCode ), strVATNumber, GetLanguage( strLanguageCode ), ref dictResults, ref WebMasterMessage ) );
      }


      public bool IsValidVAT( MemberState eState, string strVATNumber, Language eLanguage, ref Dictionary< DataItem, string > dictResults, ref string WebMasterMessage )
      {
         dictResults.Clear();
         WebMasterMessage = "";


         string responseData = "";


         try
         {
            WebRequest request = WebRequest.Create( string.Format( "http://ec.europa.eu/taxation_customs/vies/viesquer.do?locale={0}", GetLanguage( eLanguage ) ) );
            request.Method = "POST";
            request.ContentType = "application/x-www-form-urlencoded";
            
            ArrayList  queryData = new ArrayList();


            queryData.Add( string.Format("{0}={1}", "memberStateCode",          HttpUtility.UrlEncode( GetMemberState( eState ) ) ) );
            queryData.Add( string.Format("{0}={1}", "number",                   HttpUtility.UrlEncode( strVATNumber ) ) );
            queryData.Add( string.Format("{0}={1}", "traderName",               HttpUtility.UrlEncode( "" ) ) );
            queryData.Add( string.Format("{0}={1}", "traderStreet",             HttpUtility.UrlEncode( "" ) ) );
            queryData.Add( string.Format("{0}={1}", "traderPostalCode",         HttpUtility.UrlEncode( "" ) ) );
            queryData.Add( string.Format("{0}={1}", "traderCity",               HttpUtility.UrlEncode( "" ) ) );
            queryData.Add( string.Format("{0}={1}", "requesterMemberStateCode", HttpUtility.UrlEncode( RequesterStateCode ) ) );
            queryData.Add( string.Format("{0}={1}", "requesterNumber",          HttpUtility.UrlEncode( RequesterVATNumber ) ) );

            queryData.Add( string.Format("{0}={1}", "action",                   HttpUtility.UrlEncode( "check"  ) ) );
            queryData.Add( string.Format("{0}={1}", "check",                    HttpUtility.UrlEncode( "Verify" ) ) );


            string streamData = string.Join( "&", (string[])queryData.ToArray( typeof( string ) ) );
            request.ContentLength = streamData.Length;         


            using( StreamWriter sw = new StreamWriter( request.GetRequestStream() ) )
            {
               sw.Write( streamData );
            }


            HttpWebResponse response =  (HttpWebResponse)request.GetResponse();
            
            
            using( StreamReader sr = new StreamReader(response.GetResponseStream() ) )
            {
               responseData = sr.ReadToEnd().Trim();
            }
         }
         catch( Exception ex )
         {
            throw( new ServerException( ex.Message ) );
         }


         try
         {
            // Find the table with the response data.
            int posDataStart = responseData.IndexOf( "id=\"vatResponseFormTable\"", System.StringComparison.OrdinalIgnoreCase );
            if( -1 == posDataStart )
            {
               if( -1 != responseData.IndexOf( "name=\"requesterMemberStateCode\"", System.StringComparison.OrdinalIgnoreCase ) )
               {
                  throw( new Exception( ErrorMessage0 ) );
               }

               throw( new Exception( ErrorMessage1 ) );
            }


            // Find the end of the opening table tag.
            posDataStart = responseData.IndexOf( "<tr>", posDataStart + 1, System.StringComparison.OrdinalIgnoreCase );
            if( -1 == posDataStart )
            {
               throw( new Exception( ErrorMessage1 ) );
            }


            // Find the end of the table with the response data.
            int posDataEnd = responseData.IndexOf( "</table>", posDataStart + 1, System.StringComparison.OrdinalIgnoreCase );
            if( -1 == posDataEnd )
            {
               throw( new Exception( ErrorMessage1 ) );
            }


            responseData = responseData.Substring( posDataStart, posDataEnd - posDataStart ).Trim();
            

            List< string > rowData = new List< string >();


            // Split the response data by rows.
            do
            {
               // Find the end of the row.
               posDataEnd = responseData.IndexOf( "</tr>", System.StringComparison.OrdinalIgnoreCase );
               if( -1 == posDataEnd )
               {
                  throw( new Exception( ErrorMessage1 ) );
               }


               // Extract the row
               rowData.Add( responseData.Substring( 0, posDataEnd + 5 ).Trim() );


               // Remove the extracted data.
               responseData = responseData.Substring( posDataEnd + 5 ).Trim();
            }
            while( !string.IsNullOrEmpty( responseData ) );


            bool bValidStyle = false;
            bool bInvalidStyle = false;


            // Split each row to value and data.
            foreach( string rowCurrent in rowData )
            {
               List< string > RowEntries = ExtractRowData( rowCurrent.Replace( "<tr>", "" ).Replace( "</tr>", "" ).Trim() );

               if( 1 == RowEntries.Count )
               {
                  if( !bValidStyle && !bInvalidStyle && !dictResults.ContainsKey( DataItem.Status ) )
                  {
                     bValidStyle   = -1 != RowEntries[0].IndexOf( "class=\"validStyle\"",   StringComparison.OrdinalIgnoreCase );
                     bInvalidStyle = -1 != RowEntries[0].IndexOf( "class=\"invalidStyle\"", StringComparison.OrdinalIgnoreCase );

                     if( bValidStyle || bInvalidStyle )
                     {
                        string lineData = RemoveHtmlTags( RowEntries[0] ).Trim();

                        if( -1 != lineData.IndexOf( '(' ) )
                        {
                           lineData = lineData.Substring( 0, lineData.IndexOf( '(' ) ).Trim() + ".";
                        }

                        if( !string.IsNullOrEmpty( lineData ) )
                        {
                           dictResults.Add( DataItem.Status, lineData );
                        }
                     }
                  }
               }
               else
               {
                  string address = "";

                  foreach( string str in RemoveHtmlTags( RowEntries[1].Replace( "<br />", "\n" ).Replace( "<br>", "\n" ).Trim() ).Split( new char[] { '\n' } ) )
                  {
                     string part = str.Trim();

                     if( !string.IsNullOrEmpty( part ) )
                     {
                        address += ((0 == address.Length ? "" : "\n") + part);
                     }
                  }

                  dictResults.Add( (DataItem)dictResults.Count, address );
               }
            }


            if( !bValidStyle && !bInvalidStyle )
            {
               throw( new Exception( ErrorMessage8 ) );
            }

            
            if( !(bValidStyle ^ bInvalidStyle) )
            {
               throw( new Exception( ErrorMessage9 ) );
            }

               
            if( !dictResults.ContainsKey( DataItem.Status ) )
            {
               throw( new Exception( ErrorMessageA ) );
            }


            return( bValidStyle );
         }
         catch( Exception ex )
         {
            throw( new ParsingException( ex.Message ) );
         }
      }


      private List< string > ExtractRowData( string RowData )
      {
         List< string > rowData = new List< string >();


         // Split the response data by rows.
         do
         {
            // Find the end of the row.
            int posDataEnd = RowData.IndexOf( "</td>", System.StringComparison.OrdinalIgnoreCase );
            if( -1 == posDataEnd )
            {
               throw( new Exception( ErrorMessage1 ) );
            }


            // Extract the row
            rowData.Add( RowData.Substring( 0, posDataEnd + 5 ).Trim() );


            // Remove the extracted data.
            RowData = RowData.Substring( posDataEnd + 5 ).Trim();
         }
         while( !string.IsNullOrEmpty( RowData ) );
      
      
         return( rowData );
      }


      private string RemoveHtmlTags( string RawData )
      {
         int posDataStart = 0;
         int posDataEnd   = 0;

         do
         {
            posDataStart = RawData.IndexOf( '<' );
            posDataEnd   = RawData.IndexOf( '>', posDataStart + 1 );

            if( (-1 == posDataStart) ^ (-1 == posDataEnd) )
            {
               throw ( new Exception( ErrorMessage1 ) );
            }

            if( -1 != posDataStart )
            {
               RawData = RawData.Substring( 0, posDataStart ) + RawData.Substring( posDataEnd + 1 );
            }
         }
         while( (-1 != posDataStart) || (-1 != posDataEnd) );

         return( RawData );
      }
   }
}
